#include <X11/Xos.h>
#include <stdio.h>
#include <getopt.h>
#include <unistd.h>
#include <audio.h>     /* audio library header */
#include <sys/types.h> /* for unix open() */
#include <sys/stat.h>  /* for unix open() */
#include <fcntl.h>     /* for unix open() */
#include <signal.h>    /* for signal handler */
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xmu/Converters.h>

#include <X11/Xaw/Box.h>

#include "BarGraph.h"

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr
#define FFT_SIZE            1024
#define R_START               20
#define R_END                128
#define NUM_TEMPLATES          2
#define NUM_FRAMES            12
#define NUM_STAT_FRAMES       60
#define DEFAULT_THRESHOLD   16.0
#define SLIDER_MULT        100.0
#define TC_DEFAULT_THRESH    1.1
#define TC_SLIDER_MULT      10.0

char gun_strings[4][255] = {"machine gun", "rocket launcher", "plasma rifle", "bfg"};
int caught_sigint =  0;     /* completion flag */ 
float bg_thresh = DEFAULT_THRESHOLD;
float tc_thresh[NUM_TEMPLATES];
XtAppContext threshold_context;
XtAppContext tc_context[NUM_TEMPLATES];
Widget threshold_slider;
Widget tc_slider[NUM_TEMPLATES];
int tc_index = 0;

long buf[FFT_SIZE];
ALconfig config;              /* port configuration structure  */
ALport inport;                /* audio port structure          */
long nsamps = FFT_SIZE;

char write_flag = 0;  /* flags for writing the templates */
char read_flag = 0;


void threshold_slider_notify (Widget gw, XtPointer client_data, XtPointer call_data)
{
  extern float bg_thresh;
  BarGraphWidget b = (BarGraphWidget) client_data;

  bg_thresh = (float) (BarGraphGetHeight (b) * SLIDER_MULT);
  fprintf (stderr, "***BACKGROUND NOISE THRESHOLD: %lf\n", bg_thresh);
}

void tc_slider_notify (Widget gw, XtPointer client_data, XtPointer call_data)
{
  extern float tc_thresh[NUM_TEMPLATES];
  extern int tc_index;
  BarGraphWidget b = (BarGraphWidget) client_data;

  tc_thresh[tc_index] = (float) (BarGraphGetHeight (b) * TC_SLIDER_MULT);
  fprintf (stderr, "***CORRELATION THRESHOLD #%d: %lf\n", tc_index, tc_thresh[tc_index]);
}

void four1(float data[], unsigned long nn, int isign)
{
	unsigned long n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;
	float tempr,tempi;

	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) {
		if (j > i) {
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m=n >> 1;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax=2;
	while (n > mmax) {
		istep=mmax << 1;
		theta=isign*(6.28318530717959/mmax);
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;
		for (m=1;m<mmax;m+=2) {
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;
				tempr=wr*data[j]-wi*data[j+1];
				tempi=wr*data[j+1]+wi*data[j];
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr;
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
}
#undef SWAP

void
do_fft(float *fft, long *buf)
{
  static float fftbuf[FFT_SIZE * 2];
  static float mean;
  int c1;

  for(c1 = 0; c1 < FFT_SIZE; c1++)
    {
      fftbuf[2 * c1] = buf[c1];
      fftbuf[2 * c1 + 1] = 0.0;
    }
  four1(fftbuf, FFT_SIZE, 1);

  mean = 0.0;

  for(c1 = R_START; c1 < R_END; c1++)
    {
      fft[c1] = sqrt(fftbuf[2*c1] * fftbuf[2*c1] + fftbuf[2*c1 + 1] * fftbuf[2*c1 + 1]);
      mean += fft[c1] * fft[c1];
    }
  
  mean = 1 / sqrt(mean);

  for(c1 = R_START; c1 < R_END; c1++)
    {
      fft[c1] *= mean;
    }
}

void
test_for_getting_templates(float *fft, int *grab_templates)
{
  int c1;
  float temp = 0.0;

  for(c1 = R_START; c1 < R_END; c1++)
    {
      if (fft[c1] > temp)
	temp = fft[c1];
    }
  if (temp > 6.0)
    {
      fprintf(stderr, "grab %f\n", temp);
      *grab_templates = 1; 
    } 
}

float
correlate(float templates[NUM_FRAMES][FFT_SIZE], 
	  float samples[NUM_FRAMES][FFT_SIZE], int rb_head)
{
  int c1, c2;
  static float frameD;
  static float dist;
  static float tmp_corr;

  tmp_corr = 0.0;

  for (c1 = 0; c1 < NUM_FRAMES; c1++)
    {
      frameD = 0.0;
      for (c2 = R_START; c2 < R_END; c2++)
	{
	  dist = templates[c1][c2] - samples[(rb_head + c1) % NUM_FRAMES][c2];
	  dist *= dist;
	  frameD += dist;
	}
      frameD = sqrt(frameD);
      tmp_corr += frameD;
    }

  return(tmp_corr);
}  

void
flushSamplesWithBackgroundTemplate(float background[NUM_FRAMES][FFT_SIZE],
				   float samples[NUM_FRAMES][FFT_SIZE])
{
  int c1, c2;
  
  for(c1 = 0; c1 < NUM_FRAMES; c1++)
    {
      for(c2 = R_START; c2 < R_END; c2++)
	{
	  samples[c1][c2] = background[c1][c2];
	}
    }
}

void
flushSamplesWithBackgroundNoise(float samples[NUM_FRAMES][FFT_SIZE])
{
  int c1;

  for(c1 = 0; c1 < NUM_FRAMES; c1++)
    {
      ALreadsamps(inport, buf, nsamps);
      do_fft(samples[c1], buf);
    }
}

void
getBackgroundTemplate(float background[NUM_FRAMES][FFT_SIZE])
{
  int c1;

  fprintf(stderr, "*** QUIET *** CHARACTERIZING BACKGROUND NOISE ***\n");
  sleep(1);
  fprintf(stderr, "Adding frames\n");
  for(c1 = 0; c1 < NUM_FRAMES; c1++)
    {
      ALreadsamps(inport, buf, nsamps);
      do_fft(background[c1], buf);
    }
}  

void
calcBackgroundStats(float background[NUM_FRAMES][FFT_SIZE],
		    float samples[NUM_FRAMES][FFT_SIZE],
		    float *mean,
		    float *stdev)
{
  int c1;
  float temp_mean, temp_stdev, tmp;
  int rb_head = 0;
  float cor_buf[NUM_STAT_FRAMES];
  float correlation;


  flushSamplesWithBackgroundNoise(samples);

  for(c1 = 0; c1 < NUM_STAT_FRAMES; c1++)
    {
      fprintf(stderr, "Correlating frame %d of %d\n",
	      c1+1,
	      NUM_STAT_FRAMES);
      ALreadsamps(inport, buf, nsamps);
      do_fft(samples[rb_head], buf);
      rb_head = (rb_head + 1) % NUM_FRAMES;
      cor_buf[c1] = correlate(background, samples, rb_head);
    }
  
  fprintf(stderr, "Calcing back stats\n");
  temp_mean = 0.0;
  temp_stdev = 0.0;
  for(c1 = 0; c1 < NUM_STAT_FRAMES; c1++)
    {
      temp_mean += cor_buf[c1];
    }
  temp_mean /= NUM_STAT_FRAMES;
  for(c1 = 0; c1 < NUM_STAT_FRAMES; c1++)
    {
      tmp = cor_buf[c1] - temp_mean;
      tmp *= tmp;
      temp_stdev += tmp;
    }
  temp_stdev /= NUM_STAT_FRAMES;
  temp_stdev = sqrt(temp_stdev);
  fprintf(stderr, "mean: %f stdev: %f\n", temp_mean, temp_stdev);

  *mean = temp_mean;
  *stdev = temp_stdev;
}  

int
notBackgroundSound(float background[NUM_FRAMES][FFT_SIZE],
		   float samples[NUM_FRAMES][FFT_SIZE],
		   int rb_head,
		   float mean,
		   float stdev)
{
  float corr;

  corr = correlate(background, samples, rb_head);
  if (corr > mean + bg_thresh * stdev)
    {
      return(1);
    }
  return(0);
}

void
grabNewTemplates(float templates[NUM_TEMPLATES][NUM_FRAMES][FFT_SIZE],
		 float background[NUM_FRAMES][FFT_SIZE],
		 float samples[NUM_FRAMES][FFT_SIZE],
		 float bg_mean,
		 float bg_stdev)
{
  int c1, c2, c3;
  int rb_head = 0;

  flushSamplesWithBackgroundNoise(samples);

  for(c1 = 0; c1 < NUM_TEMPLATES; c1++)
    {
      do
	{
	  ALreadsamps(inport, buf, nsamps);
	  do_fft(samples[rb_head], buf);
	  rb_head = (rb_head + 1) % NUM_FRAMES;
	}
      while (!notBackgroundSound(background,
				 samples,
				 rb_head,
				 bg_mean,
				 bg_stdev));

      fprintf(stderr, "Grabbing template %d of %d\n",
	      c1+1,
	      NUM_TEMPLATES);

      for(c2 = 0; c2 < NUM_FRAMES; c2++)
	{
	  for(c3 = 0; c3 < FFT_SIZE; c3++)
	    {
	      templates[c1][c2][c3] = samples[(rb_head + c2) % NUM_FRAMES][c3];
	    }
	}


/*      
      for(c2 = 0; c2 < NUM_FRAMES; c2++)
	{
	  ALreadsamps(inport, buf, nsamps);
	  do_fft(templates[c1][c2], buf);
	}
*/
      
      fprintf(stderr, "Pausing between grabs\n");
      sleep(4);
      fprintf(stderr, "Flushing sample buffer\n");
      flushSamplesWithBackgroundNoise(samples);
      rb_head = 0;

    }
}
	
void
correlateTemplatesWithBackground(float templates[NUM_TEMPLATES][NUM_FRAMES][FFT_SIZE],
				 float samples[NUM_FRAMES][FFT_SIZE],
				 float tc_mean[NUM_TEMPLATES],
				 float tc_stdev[NUM_TEMPLATES])
{
  float tc_buf[NUM_TEMPLATES][NUM_STAT_FRAMES];
  int c1, c2;
  int rb_head = 0;
  float temp;

  flushSamplesWithBackgroundNoise(samples);

  for(c1 = 0; c1 < NUM_STAT_FRAMES; c1++)
    {
      ALreadsamps(inport, buf, nsamps);
      do_fft(samples[rb_head], buf);
      rb_head = (rb_head + 1) % NUM_FRAMES;

      fprintf(stderr, "Correlating templates, frame %d of %d\n",
	      c1+1,
	      NUM_STAT_FRAMES);
      for(c2 = 0; c2 < NUM_TEMPLATES; c2++)
	{
	  tc_buf[c2][c1] = correlate(templates[c2],
				     samples,
				     rb_head);
	}
    }

  for(c1 = 0; c1 < NUM_TEMPLATES; c1++)
    {
      tc_mean[c1] = 0.0;
      tc_stdev[c1] = 0.0;

      for(c2 = 0; c2 < NUM_STAT_FRAMES; c2++)
	{
	  tc_mean[c1] += tc_buf[c1][c2];
	}

      tc_mean[c1] /= NUM_STAT_FRAMES;
    }

  for(c1 = 0; c1 < NUM_TEMPLATES; c1++)
    {
      for(c2 = 0; c2 < NUM_STAT_FRAMES; c2++)
	{
	  temp = tc_buf[c1][c2] - tc_mean[c1];
	  temp *= temp;
	  tc_stdev[c1] += temp;
	}

      tc_stdev[c1] /= NUM_STAT_FRAMES;
      tc_stdev[c1] = sqrt(tc_stdev[c1]);
    }
}

int
doTemplateMatch(float templates[NUM_TEMPLATES][NUM_FRAMES][FFT_SIZE],
		float samples[NUM_FRAMES][FFT_SIZE],
		int rb_head,
		float tc_mean[NUM_TEMPLATES],
		float tc_stdev[NUM_TEMPLATES])
{
  int c1;
  int number_matched;
  float temp_distance;
  float temp_corr;

/*  temp_distance = correlate(templates[0], samples, rb_head); */
  number_matched = -1;

  for(c1 = 0; c1 < NUM_TEMPLATES; c1++)
    {
      temp_corr = correlate(templates[c1], samples, rb_head);

      if (temp_corr < tc_mean[c1] - tc_stdev[c1] * tc_thresh[c1])
	{
	  if (number_matched >= 0)  /* We already had a match */
	    {
	      if (temp_corr < temp_distance)
		{
		  number_matched = c1;
		  temp_distance = temp_corr;
		}
	    }
	  else
	    {
	      number_matched = c1;
	      temp_distance = temp_corr;
	    }
	}
    }

/*
      if (temp_corr < temp_distance)
	{
	  temp_distance = temp_corr;
	  number_matched = c1;
	}
    }
*/

  return(number_matched);
}

void
readTemplates(float *bg_thresh,
	      float templates[NUM_TEMPLATES][NUM_FRAMES][FFT_SIZE],
	      float tc_thresh[NUM_TEMPLATES])     
{
  FILE *fp;
  int fd;
  char filename[255];
  int counter;
  int c1, c2;
  int failure_flag = 0;

  sprintf(filename, "templates.dat");

  fp = fopen(filename, "r");
  
  fscanf(fp, "%d\n", &counter);  /* NUM_TEMPLATES */
  if (counter != NUM_TEMPLATES)
    {
      fprintf(stderr, "Error: unexpected number of templates\n");
      failure_flag = 1;
    }
  else
    {
      fscanf(fp, "%d\n", &counter);  /* NUM_FRAMES */
      if (counter != NUM_FRAMES)
	{
	  fprintf(stderr, "Error: unexpected number of frames\n");
	  failure_flag = 1;
	}
      else
	{
	  fscanf(fp, "%d\n", &counter);  /* FFT_SIZE */
	  if(counter != FFT_SIZE)
	    {
	      fprintf(stderr, "Error: unexpected fft size\n");
	      failure_flag = 1;
	    }
	  else
	    {
	      fscanf(fp, "%f\n", bg_thresh);
	      for(counter = 0; counter < NUM_TEMPLATES; counter++)
		{
		  fscanf(fp, "%f\n", &(tc_thresh[counter]));
		}
	    }
	}
    }

  fclose(fp);

  if (!failure_flag)
    {
      for(counter = 0; counter < NUM_TEMPLATES; counter++)
	{
	  sprintf(filename, "template%03d.fft", counter);
	  fd = open(filename, O_RDONLY);

	  read(fd, (char *)templates[counter], NUM_FRAMES*FFT_SIZE*sizeof(float));

	  close(fd);
	}
    }
}

void
writeTemplates(float bg_thresh,
	       float templates[NUM_TEMPLATES][NUM_FRAMES][FFT_SIZE],
	       float tc_thresh[NUM_TEMPLATES])
{
  FILE *fp;
  int fd;
  char filename[255];
  int counter;

  sprintf(filename, "templates.dat");

  fp = fopen(filename, "w");

  fprintf(fp, "%d\n", NUM_TEMPLATES);
  fprintf(fp, "%d\n", NUM_FRAMES);
  fprintf(fp, "%d\n", FFT_SIZE);
  fprintf(fp, "%f\n", bg_thresh);

  for(counter = 0; counter < NUM_TEMPLATES; counter++)
    {
      fprintf(fp, "%f\n", tc_thresh[counter]);
    }
  
  fclose(fp);

  for(counter = 0; counter < NUM_TEMPLATES; counter++)
    {
      sprintf(filename, "template%03d.fft", counter);
      fd = open(filename, O_CREAT| O_RDWR | O_TRUNC);
      fchmod(fd, 0666);

      write(fd, (char *)templates[counter], NUM_FRAMES*FFT_SIZE*sizeof(float));

      close(fd);
    }
}

void 
catch_sigint()
{
  caught_sigint=1;
}

void
usage()
{
  fprintf(stderr, "Usage: fftsample [-r] [-w]\n");
  fprintf(stderr, "-r: read templates from disk\n");
  fprintf(stderr, "-w: write templates to disk upon exiting\n");
}

int
main(int argc, char **argv)
{
  int grab_templates = 0, got_templates = 0;
  int sc;                       /* # of frames grabbed */
  int rb_head;                  /* index head of ring buffer     */
  int fd = 0;                   /* output file                   */
  float fft[FFT_SIZE];
  float templates[NUM_TEMPLATES][NUM_FRAMES][FFT_SIZE];
  float templates2[NUM_TEMPLATES][NUM_FRAMES][FFT_SIZE];
  float samples[NUM_FRAMES][FFT_SIZE];
  float background[NUM_FRAMES][FFT_SIZE];
  float tc_mean[NUM_TEMPLATES];
  float tc_stdev[NUM_TEMPLATES];
  String title;
  int tm;

  long PVbuffer[4];
  float correlation;
  int c1;
  float cor_buf[NUM_STAT_FRAMES];
  float bg_mean, bg_stdev, temp_stdev;
  int counter = 0;
  int i;
  int c;

  int c2, c3;

/* Bar graph stuff */

  extern XtAppContext threshold_context;
  Widget shell1, box1;
  extern Widget threshold_slider;
  XEvent event;

  Widget shell[NUM_TEMPLATES], box[NUM_TEMPLATES];

  while ((c = getopt(argc, argv, "rwh")) != EOF) 
    {
      switch (c) 
	{
	case 'h':
	  usage();
	  break;
	  
	case 'r':
	  read_flag = 1;
	  break;
	  
	case 'w':
	  write_flag = 1;
	  break;
	}
    }

  i = 1;
  title = "BackgroundSlider";
  shell1 = XtAppInitialize (&threshold_context, "ThresholdSlider", NULL, 0, &i , &title, NULL, NULL, 0);
  box1 = XtCreateManagedWidget ("box", boxWidgetClass, shell1, NULL, 0);
  threshold_slider = XtCreateManagedWidget ("threshold slider", barGraphWidgetClass, box1, NULL, 0);

  XtAddCallback (threshold_slider, XtNplacementCallback, threshold_slider_notify, threshold_slider);
  XtAddCallback (threshold_slider, XtNmotionCallback, threshold_slider_notify, threshold_slider);

  BarGraphSetHeight ((BarGraphWidget) threshold_slider, bg_thresh / SLIDER_MULT);

  XtRealizeWidget (shell1);

  tc_thresh[0] = 2.75;
  tc_thresh[1] = 1.30;

  for(tc_index=0; tc_index<NUM_TEMPLATES; tc_index++)
    {
      sprintf(title, "tc%d", tc_index);
      shell[tc_index] = XtAppInitialize(&tc_context[tc_index], "CorrelationSlider", NULL, 0, &i, &title, NULL, NULL, 0);
      box[tc_index] = XtCreateManagedWidget("box", boxWidgetClass, shell[tc_index], NULL, 0);
      tc_slider[tc_index] = XtCreateManagedWidget("tc slider", barGraphWidgetClass, box[tc_index], NULL, 0);

      XtAddCallback (tc_slider[tc_index], XtNplacementCallback, tc_slider_notify, tc_slider[tc_index]);
      XtAddCallback (tc_slider[tc_index], XtNmotionCallback, tc_slider_notify, tc_slider[tc_index]);

      BarGraphSetHeight((BarGraphWidget) tc_slider[tc_index], tc_thresh[tc_index] / TC_SLIDER_MULT);

      XtRealizeWidget(shell[tc_index]);
    }

  while(XtAppPending(threshold_context))
    {
      XtAppNextEvent(threshold_context, &event);
      XtDispatchEvent(&event); 
    }

  for(tc_index = 0; tc_index < NUM_TEMPLATES; tc_index++)
    {
      while(XtAppPending(tc_context[tc_index]))
	{
	  XtAppNextEvent(tc_context[tc_index], &event);
	  XtDispatchEvent(&event); 
	}
    }

  bzero((void *)samples, NUM_FRAMES * FFT_SIZE * sizeof(float));
  config = ALnewconfig();
  PVbuffer[0] = AL_INPUT_SOURCE;
  PVbuffer[1] = AL_INPUT_MIC;
  PVbuffer[2] = AL_INPUT_RATE;
  PVbuffer[3] = AL_RATE_32000;

  ALsetparams(AL_DEFAULT_DEVICE, PVbuffer, 4);
  ALsetqueuesize(config, 2 * FFT_SIZE);
  ALsetwidth(config, AL_SAMPLE_24);
  ALsetchannels(config, AL_MONO);
  sigset(SIGINT, catch_sigint);
  
  inport = ALopenport("inport", "r", config); 
  
  if (!read_flag)
    {
      getBackgroundTemplate(background);
      calcBackgroundStats(background,
			  samples,
			  &bg_mean,
			  &bg_stdev);
      grabNewTemplates(templates,
		       background,
		       samples,
		       bg_mean,
		       bg_stdev);
      correlateTemplatesWithBackground(templates,
				       samples,
				       tc_mean,
				       tc_stdev);
    }
  else
    {
      getBackgroundTemplate(background);
      calcBackgroundStats(background,
			  samples,
			  &bg_mean,
			  &bg_stdev);
      readTemplates(&bg_thresh,
		    templates,
		    tc_thresh);
      correlateTemplatesWithBackground(templates,
				       samples,
				       tc_mean,
				       tc_stdev);
    }

  /* test writing and reading */
/*
  writeTemplates(bg_thresh, templates, tc_thresh);
  readTemplates(&bg_thresh, templates2, tc_thresh);

  for (c1 = 0; c1 < NUM_TEMPLATES; c1++)
    {
      for (c2 = 0; c2 < NUM_FRAMES; c2++)
	{
	  for (c3 = 0; c3 < FFT_SIZE; c3++)
	    {
	      if (templates[c1][c2][c3] != templates2[c1][c2][c3])
		{
		  fprintf(stderr, "BUG! %f != %f\n", templates[c1][c2][c3], templates2[c1][c2][c3]);
		}
	    }
	}
    }
*/
  while (!caught_sigint)
    {
      ALreadsamps(inport, buf, nsamps);
      do_fft(samples[rb_head], buf);
      rb_head = (rb_head + 1) % NUM_FRAMES;
      if (notBackgroundSound(background,
			     samples,
			     rb_head,
			     bg_mean,
			     bg_stdev))
	{
	  tm = doTemplateMatch(templates, 
			       samples,
			       rb_head,
			       tc_mean,
			       tc_stdev);
	  switch(tm)
	    {
	    case -1:
	      fprintf(stdout, "0");
/*	      fprintf(stderr, "fired weapon\n"); 
*/	      break;
	    case 0:
	      fprintf(stdout, "4");
/*	      fprintf(stderr, "switched to weapon 4\n");
*/	      break;
	    case 1:
	      fprintf(stdout, "5");
/*	      fprintf(stderr, "switched to weapon 5\n");
*/	      break;
	    }
	  fflush(stdout);

/*
	  if (tm >= 0)
	    {
	      fprintf(stderr, "%s\n", gun_strings[tm]);
	    }
	  else
	    {
	      fprintf(stderr, "No match\n");
	    }
*/

	}


/*	  flushSamplesWithBackgroundTemplate(background, samples);
	  sleep(1);
*/

      while(XtAppPending(threshold_context))
	{
	  XtAppNextEvent(threshold_context, &event);
	  XtDispatchEvent(&event); 
	}
      
      BarGraphSetHeight ((BarGraphWidget) threshold_slider, bg_thresh / SLIDER_MULT);

      for(tc_index = 0; tc_index < NUM_TEMPLATES; tc_index++)
	{
	  while(XtAppPending(tc_context[tc_index]))
	    {
	      XtAppNextEvent(tc_context[tc_index], &event);
	      XtDispatchEvent(&event); 
	    }
	  BarGraphSetHeight((BarGraphWidget) tc_slider[tc_index], tc_thresh[tc_index] / TC_SLIDER_MULT);
	}

    }
      

/*
  while (!caught_sigint)
    {  
      ALreadsamps(inport, buf, nsamps);
      do_fft(fft, buf);
      if (!grab_templates)
	test_for_getting_templates(fft, &grab_templates);
      if (grab_templates && !got_templates)
	{
	  fprintf(stderr, "Grabbing templates\n");
	  got_templates = 1;
	  for(c1 = 0; c1 < NUM_FRAMES; c1++)
	    {
	      ALreadsamps(inport, buf, nsamps);
	      do_fft(templates[c1], buf);
	    }
	  fprintf(stderr, "Going to sleep\n");
	  sleep(1);
	  fprintf(stderr, "Waking up\n");
	}
      if (got_templates)
	{
	  ALreadsamps(inport, buf, nsamps);
	  do_fft(samples[rb_head], buf);
	  rb_head = (rb_head + 1) % NUM_FRAMES;
	  correlate(templates, samples, rb_head, &correlation);
	  if (counter < NUM_STAT_FRAMES)
	    {
	      cor_buf[counter] = correlation;
	      counter++;
	    }
	  if (counter == NUM_STAT_FRAMES)
	    {
	      fprintf(stderr, "Calcing back stats\n");
	      counter++;
	      mean = 0.0;
	      stdev = 0.0;
	      for(c1 = 0; c1 < NUM_STAT_FRAMES; c1++)
		{
		  mean += cor_buf[c1];
		}
	      mean /= NUM_STAT_FRAMES;
	      for(c1 = 0; c1 < NUM_STAT_FRAMES; c1++)
		{
		  temp_stdev = cor_buf[c1] - mean;
		  temp_stdev *= temp_stdev;
		  stdev += temp_stdev;
		}
	      stdev /= NUM_STAT_FRAMES;
	      stdev = sqrt(stdev);
	      fprintf(stderr, "mean: %f stdev: %f\n", mean, stdev);
	    }
	  fprintf(stderr, "Correlation score: %f\n", correlation);
	  if (correlation < mean - bg_thresh * stdev)
	    fprintf(stderr, "OOOOOH YES\n");

	  while(XtAppPending(threshold_context))
	    {
	      XtAppNextEvent(threshold_context, &event);
	      XtDispatchEvent(&event); 
	    }

	  BarGraphSetHeight ((BarGraphWidget) threshold_slider, bg_thresh / SLIDER_MULT);
	}
    }
*/

  if (write_flag)
    writeTemplates(bg_thresh, templates, tc_thresh);

  ALfreeconfig(config);               /* free the config object    */
  ALcloseport(inport);                /* shut down the input port  */

  return 0;
/*  close(fd);
*/
}
